@@ -18,7 +18,7 @@ import webbrowser
 from datetime import timedelta
 from PyQt5.QtCore import Qt, QCoreApplication, QDate, QDateTime, QFileSelector, QMargins, pyqtSignal, pyqtSlot
 from PyQt5.QtGui import QColor, QCursor, QFont, QGuiApplication, QIcon, QPen
-from PyQt5.QtWidgets import QAction, QDialog, QMenu, QMessageBox
+from PyQt5.QtWidgets import QAction, QDialog, QInputDialog, QLineEdit, QMenu, QMessageBox
 from QCustomPlot2 import (QCP, QCustomPlot, QCPAxisTickerDateTime, QCPAxisTickerFixed, QCPDataRange, QCPDataSelection,
                           QCPGraph, QCPItemPosition, QCPItemText, QCPItemTracer, QCPScatterStyle, QCPTextElement)
 import gpxviewer.gpxmodel as gpx
@@ -31,6 +31,22 @@ class PlotCanvas(QCustomPlot):
   def __init__(self, parent=None):
     super(QCustomPlot, self).__init__(parent)
 
+    self.showInfo = False
+    self.moveCaption = False
+    self.font = QFont()
+    self.font.setStyleHint(QFont.SansSerif)
+    self.themeSelector = QFileSelector()
+    self.themeSelector.setExtraSelectors([TheConfig['MainWindow']['ColorTheme']])
+
+    self.addLayer('profile', self.layer('main'), QCustomPlot.limBelow)
+
+    self.setInteraction(QCP.iSelectItems)
+    self.setInteraction(QCP.iSelectPlottables)
+    self.setInteraction(QCP.iRangeDrag)
+    self.setInteraction(QCP.iRangeZoom)
+    self.axisRect().setRangeDrag(Qt.Horizontal)
+    self.axisRect().setRangeZoom(Qt.Horizontal)
+
     self.axisRect().setupFullAxesBox(True)
     self.xAxis.setTickLabelPadding(10)
     self.yAxis.setTickLabelPadding(10)
@@ -43,35 +59,71 @@ class PlotCanvas(QCustomPlot):
     self.yAxis.setTicker(self.yTicker)
     self.yAxis2.setTicker(self.yTicker)
 
-    self.addLayer('profile', self.layer('main'), QCustomPlot.limBelow)
+    self.setAutoAddPlottableToLegend(False)
+    self.legendTitle = QCPTextElement(self)
+    self.xLegendText = QCPTextElement(self)
+    self.yLegendText = QCPTextElement(self)
+    self.legendTitle.setMinimumSize(200, 0)
+    self.xLegendText.setMinimumSize(200, 0)
+    self.yLegendText.setMinimumSize(200, 0)
+    self.legendTitle.setLayer(self.legend.layer())
+    self.xLegendText.setLayer(self.legend.layer())
+    self.yLegendText.setLayer(self.legend.layer())
+    self.legend.addElement(self.legendTitle)
+    self.legend.addElement(self.xLegendText)
+    self.legend.addElement(self.yLegendText)
+    self.legendTitle.setMargins(QMargins(10, 5, 10, 5))
+    self.xLegendText.setMargins(QMargins(10, 5, 10, 5))
+    self.yLegendText.setMargins(QMargins(10, 5, 10, 5))
 
-    self.setInteraction(QCP.iSelectItems)
-    self.setInteraction(QCP.iSelectPlottables)
-    self.setInteraction(QCP.iRangeDrag)
-    self.setInteraction(QCP.iRangeZoom)
-    self.axisRect().setRangeDrag(Qt.Horizontal)
-    self.axisRect().setRangeZoom(Qt.Horizontal)
+    self.initShortcuts()
 
     self.selectionChangedByUser.connect(self.onSelectionChanged)
 
-    self.font = QFont()
-    self.font.setStyleHint(QFont.SansSerif)
+  def initShortcuts(self):
+    self.actMarker = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-marker.svg')), self.tr('Point with marker'), self)
+    self.actMarker.setShortcut(Qt.Key_M)
+    self.actMarker.setCheckable(True)
+    self.actMarker.triggered.connect(self.onMarkerPoints)
+    self.addAction(self.actMarker)
 
-    self.themeSelector = QFileSelector()
-    self.themeSelector.setExtraSelectors([TheConfig['MainWindow']['ColorTheme']])
+    self.actCaption = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-caption.svg')), self.tr('Point with caption'), self)
+    self.actCaption.setShortcut(Qt.Key_C)
+    self.actCaption.setCheckable(True)
+    self.actCaption.triggered.connect(self.onCaptionPoints)
+    self.addAction(self.actCaption)
 
-    self.showInfo = False
-    self.setAutoAddPlottableToLegend(False)
-    self.xLegendText = QCPTextElement(self)
-    self.yLegendText = QCPTextElement(self)
-    self.xLegendText.setMinimumSize(200, 0)
-    self.yLegendText.setMinimumSize(200, 0)
-    self.xLegendText.setLayer(self.legend.layer())
-    self.yLegendText.setLayer(self.legend.layer())
-    self.legend.addElement(self.xLegendText)
-    self.legend.addElement(self.yLegendText)
-    self.xLegendText.setMargins(QMargins(10, 5, 10, 5))
-    self.yLegendText.setMargins(QMargins(10, 5, 10, 5))
+    self.actSplit = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-splitline.svg')), self.tr('Point with splitting line'), self)
+    self.actSplit.setShortcut(Qt.Key_S)
+    self.actSplit.setCheckable(True)
+    self.actSplit.triggered.connect(self.onSplitLines)
+    self.addAction(self.actSplit)
+
+    self.actNeglect = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-neglect-distance.svg')), self.tr('Neglect previous distance'), self)
+    self.actNeglect.setShortcut(Qt.Key_N)
+    self.actNeglect.setCheckable(True)
+    self.actNeglect.triggered.connect(self.onNeglectDistance)
+    self.addAction(self.actNeglect)
+
+    self.actSkip = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-skip.svg')), self.tr('Skip point'), self)
+    self.actSkip.setShortcut(Qt.Key_Delete)
+    self.actSkip.setCheckable(True)
+    self.actSkip.triggered.connect(self.onSkipPoints)
+    self.addAction(self.actSkip)
+
+    self.actRename = QAction(QIcon(self.themeSelector.select(':/icons/edit-rename.svg')), self.tr('Rename...'), self)
+    self.actRename.setShortcut(Qt.Key_F2)
+    self.actRename.triggered.connect(self.onRenamePoints)
+    self.addAction(self.actRename)
+
+    self.actResetName = QAction(QIcon(self.themeSelector.select(':/icons/edit-clear.svg')), self.tr('Reset name'), self)
+    self.actResetName.triggered.connect(self.onResetPointNames)
+    self.addAction(self.actResetName)
+
+    self.actStyle = QAction(QIcon(self.themeSelector.select(':/icons/configure.svg')), self.tr('Point style'), self)
+    self.actStyle.setShortcut(Qt.ALT + Qt.Key_Return)
+    self.actStyle.triggered.connect(self.onPointStyle)
+    self.addAction(self.actStyle)
 
   def deselectAll(self):
     super(PlotCanvas, self).deselectAll()
@@ -95,7 +147,6 @@ class PlotCanvas(QCustomPlot):
     self.xx = []
     self.yy = []
     self.markers = []
-    self.infoMarkers = []
     self.captions = []
     self.splitLines = []
     self.neglectPoints = []
@@ -126,11 +177,9 @@ class PlotCanvas(QCustomPlot):
             self.neglectPoints += [self.xx[-1]]
           if TheDocument.wptmodel.index(p, 0).data(gpx.SplitLineRole):
             self.splitLines += [SplitLine(self, p, self.xx[-1], self.yy[-1])]
-          if TheDocument.wptmodel.index(p, 0).data(gpx.MarkerRole):
-            self.markers += [MarkerItem(self, p, self.xx[-1], self.yy[-1])]
-          else:
-            self.infoMarkers += [MarkerItem(self, p, self.xx[-1], self.yy[-1])]
-            self.infoMarkers[-1].setVisible(False)
+          self.markers += [MarkerItem(self, p, self.xx[-1], self.yy[-1])]
+          if not TheDocument.wptmodel.index(p, 0).data(gpx.MarkerRole):
+            self.markers[-1].setVisible(False)
           if TheDocument.wptmodel.index(p, 0).data(gpx.CaptionRole):
             self.captions += [CaptionItem(self, p, self.xx[-1], self.yy[-1])]
       elif p[0] in self.trkRows or len(self.trkRows) == 0 and len(self.wptRows) == 0:  # tracks
@@ -187,10 +236,11 @@ class PlotCanvas(QCustomPlot):
     self.yAxis.setTickLabelFont(self.font)
     self.yAxis.setLabelFont(self.font)
 
-    for text in {self.xLegendText, self.yLegendText}:
+    for text in {self.legendTitle, self.xLegendText, self.yLegendText}:
       text.setFont(self.font)
       text.setTextColor(textColor)
 
+
     if self.column == gpx.TIME:
       self.xAxis.setTickLabelRotation(-90)
     else:
@@ -253,6 +303,7 @@ class PlotCanvas(QCustomPlot):
   def showInformation(self, enable):
     if enable:
       self.tracer = QCPItemTracer(self)
+      self.tracer.setInterpolating(True)
       self.tracer.setSelectable(False)
       lineColor = QGuiApplication.palette().text().color() if TheConfig.getValue('ProfileStyle', 'UseSystemTheme') else QColor(Qt.black)
       self.tracer.setPen(QPen(lineColor))
@@ -260,9 +311,13 @@ class PlotCanvas(QCustomPlot):
     self.showInfo = enable
     self.tracer.setVisible(enable)
     self.legend.setVisible(enable)
-    for m in self.infoMarkers:
-      m.setVisible(enable)
-      m.setSelectable(enable)
+
+    markedPoints = TheDocument.wptmodel.getMarkedPoints()
+    for m in self.markers:
+      if m.idx not in markedPoints:
+        m.setVisible(enable)
+        m.setSelectable(enable)
+
     self.deselectAll()
     self.replot()
 
@@ -274,64 +329,85 @@ class PlotCanvas(QCustomPlot):
       else:
         super(PlotCanvas, self).keyPressEvent(event)
 
-    elif event.key() == Qt.Key_Menu and self.selectedElement is not None:
+    elif event.key() == Qt.Key_Menu and event.modifiers() == Qt.NoModifier and self.selectedElement is not None:
       self.contextMenu()
 
-    elif event.key() in {Qt.Key_Left, Qt.Key_Right, Qt.Key_Down, Qt.Key_Up} and len(self.selectedItems()) == 1:
+    # Move caption
+    elif event.key() in {Qt.Key_Left, Qt.Key_Right, Qt.Key_Down, Qt.Key_Up} and type(self.selectedElement) == CaptionItem:
       x = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.CaptionStyleRole)[gpx.CAPTION_POSX]
       y = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.CaptionStyleRole)[gpx.CAPTION_POSY]
+      d = 10 if event.modifiers() == Qt.ControlModifier else 1
       if event.key() == Qt.Key_Left:
-        TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSX, x - 1)
+        TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSX, x - d)
       elif event.key() == Qt.Key_Right:
-        TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSX, x + 1)
+        TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSX, x + d)
       elif event.key() == Qt.Key_Down:
-        TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSY, y - 1)
+        TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSY, y - d)
       elif event.key() == Qt.Key_Up:
-        TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSY, y + 1)
+        TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSY, y + d)
+      self.replot()
+
+    # Select previous/next marker
+    elif event.key() in {Qt.Key_Left, Qt.Key_Right} and type(self.selectedElement) == MarkerItem:
+      step = 1 if event.key() == Qt.Key_Right else -1
+      ind = [m.idx for m in self.markers].index(self.selectedElement.idx)
+      self.markers[ind].setSelection(QCPDataSelection())
+
+      while True:
+        ind += step
+        if (not 0 < ind < len(self.markers) - 1) or TheDocument.wptmodel.index(self.markers[ind].idx, gpx.NAME).data(gpx.MarkerRole) or \
+           (self.showInfo and event.modifiers() != Qt.ControlModifier):
+          break
+      ind = min(max(ind, 0), len(self.markers) - 1)
+
+      self.markers[ind].setSelection(QCPDataSelection(QCPDataRange(0, 1)))
+      self.onSelectionChanged()
       self.replot()
 
     else:
       super(PlotCanvas, self).keyPressEvent(event)
 
   def mouseMoveEvent(self, event):
-    if self.axisRect().rect().contains(event.pos()):
-      # Show information
-      if self.showInfo:
-        if self.plottableAt(event.pos(), True) is None and self.itemAt(event.pos(), True) is None:
-          self.setCursor(QCursor(Qt.CrossCursor))
-        else:
-          self.setCursor(QCursor())
-
-        for profile in self.layer('profile').children():
-          if profile.getKeyRange()[0].contains(self.xAxis.pixelToCoord(event.pos().x())):
-            self.tracer.setGraph(profile)
-        self.tracer.setGraphKey(self.xAxis.pixelToCoord(event.pos().x()))
-        self.replot()
-        if self.column == gpx.DIST:
-          self.xLegendText.setText(self.tr('Distance: ') + str(round(self.tracer.position.key(), 3)))
-        elif self.column == gpx.TIME_DAYS:
-          self.xLegendText.setText(self.tr('Time: ') + str(round(self.tracer.position.key(), 3)))
-        else:  # absolute time
-          self.xLegendText.setText(self.tr('Time: ') +
-                                   QCPAxisTickerDateTime.keyToDateTime(self.tracer.position.key()).toString('yyyy-MM-dd HH:mm:ss'))
-        self.yLegendText.setText(self.tr('Altitude: ') + str(round(self.tracer.position.value())))
+    # Show information
+    if self.showInfo and self.axisRect().rect().contains(event.pos()):
+      for profile in self.layer('profile').children():
+        if profile.getKeyRange()[0].contains(self.xAxis.pixelToCoord(event.pos().x())):
+          self.tracer.setGraph(profile)
+      self.tracer.setGraphKey(self.xAxis.pixelToCoord(event.pos().x()))
+      if self.selectedElement is None:
+        self.updateLegend()
+      else:
         self.replot()
 
-      else:  # not showing information
-        if self.plottableAt(event.pos(), True) is None and self.itemAt(event.pos(), True) is None:
-          self.setCursor(QCursor(Qt.OpenHandCursor))
-        else:
-          self.setCursor(QCursor())
+    self.updateCursorShape(event.pos())
 
-    # Outside the axes rect
-    elif event.pos().x() < self.axisRect().rect().left():
-      self.setCursor(QCursor(Qt.SizeVerCursor))
-    else:
-      self.setCursor(QCursor())
+    # Move caption
+    if self.moveCaption and QGuiApplication.mouseButtons() == Qt.LeftButton:
+      x = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.CaptionStyleRole)[gpx.CAPTION_POSX]
+      y = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.CaptionStyleRole)[gpx.CAPTION_POSY]
+      delta = event.pos() - self.prevPos
+      TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSX, x + delta.x())
+      TheDocument.wptmodel.setPointStyle([self.selectedElement.idx], gpx.CAPTION_POSY, y - delta.y())
+      self.replot()
+    else:  # no caption is selected
+      super(PlotCanvas, self).mouseMoveEvent(event)
 
-    super(PlotCanvas, self).mouseMoveEvent(event)
+    self.prevPos = event.pos()
+
+  def mousePressEvent(self, event):
+    if event.button() == Qt.LeftButton:
+      if self.plottableAt(event.pos(), True) is None and self.itemAt(event.pos(), True) is None:
+        self.setCursor(QCursor(Qt.ClosedHandCursor))
+
+      if type(self.selectedElement) == CaptionItem and self.itemAt(event.pos(), True) == self.selectedElement:
+        self.moveCaption = True
+
+    super(PlotCanvas, self).mousePressEvent(event)
 
   def mouseReleaseEvent(self, event):
+    if event.button() == Qt.LeftButton:
+      self.updateCursorShape(event.pos())
+
     if event.button() == Qt.RightButton:
       self.deselectAll()
       item = self.itemAt(event.pos(), True)
@@ -343,9 +419,11 @@ class PlotCanvas(QCustomPlot):
         self.contextMenu()
       self.replot()
 
+    self.moveCaption = False
     super(PlotCanvas, self).mouseReleaseEvent(event)
 
   def wheelEvent(self, event):
+    # Outside the axes rect
     if event.pos().x() < self.axisRect().rect().left():
       if event.pos().y() < self.axisRect().rect().center().y():
         self.maxalt = max(self.maxalt + (100 if event.angleDelta().y() > 0 else -100), self.minalt + 100)
@@ -361,34 +439,59 @@ class PlotCanvas(QCustomPlot):
     else:  # inside the axes rect
       super(PlotCanvas, self).wheelEvent(event)
 
+  def updateCursorShape(self, pos):
+    if QGuiApplication.mouseButtons() == Qt.LeftButton:
+      self.setCursor(QCursor(Qt.ClosedHandCursor))
+    else:
+      # Inside the axes rect
+      if self.axisRect().rect().contains(pos):
+        if self.plottableAt(pos, True) is None and self.itemAt(pos, True) is None:
+          self.setCursor(QCursor(Qt.CrossCursor if self.showInfo else Qt.OpenHandCursor))
+        else:
+          self.setCursor(QCursor(Qt.PointingHandCursor))
+      else:  # outside the axes rect
+        self.setCursor(QCursor(Qt.SizeVerCursor if pos.x() < self.axisRect().rect().left() else Qt.ArrowCursor))
+
+  def updateLegend(self):
+    if self.selectedElement is not None:
+      self.legendTitle.setText(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data())
+      xTitle = self.tr('Distance: ') if self.column == gpx.DIST else self.tr('Time: ')
+      self.xLegendText.setText(xTitle + TheDocument.wptmodel.index(self.selectedElement.idx, self.column).data())
+      self.yLegendText.setText(self.tr('Altitude: ') + TheDocument.wptmodel.index(self.selectedElement.idx, gpx.ALT).data())
+
+    else:  # no item is selected
+      self.legendTitle.setText(self.tr('Cursor'))
+      if self.column == gpx.DIST:
+        self.xLegendText.setText(self.tr('Distance: ') + str(round(self.tracer.position.key(), 3)))
+      elif self.column == gpx.TIME_DAYS:
+        self.xLegendText.setText(self.tr('Time: ') + str(round(self.tracer.position.key(), 3)))
+      else:  # absolute time
+        self.xLegendText.setText(self.tr('Time: ') +
+                                 QCPAxisTickerDateTime.keyToDateTime(self.tracer.position.key()).toString('yyyy-MM-dd HH:mm:ss'))
+      self.yLegendText.setText(self.tr('Altitude: ') + str(round(self.tracer.position.value())))
+
+    self.replot()
+
   def contextMenu(self):
-    actMarker = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-marker.svg')), self.tr('Point with marker'), self)
-    actMarker.setCheckable(True)
-    actMarker.setChecked(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.MarkerRole))
-    actMarker.triggered.connect(self.onMarkerPoints)
-    actCaption = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-caption.svg')), self.tr('Point with caption'), self)
-    actCaption.setCheckable(True)
-    actCaption.setChecked(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.CaptionRole))
-    actCaption.triggered.connect(self.onCaptionPoints)
-    actSplit = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-splitline.svg')), self.tr('Point with splitting line'), self)
-    actSplit.setCheckable(True)
-    actSplit.setChecked(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.SplitLineRole))
-    actSplit.triggered.connect(self.onSplitLines)
-    actSkip = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-skip.svg')), self.tr('Skip point'), self)
-    actSkip.setCheckable(True)
-    actSkip.setChecked(not TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.IncludeRole))
-    actSkip.triggered.connect(self.onSkipPoints)
-    actStyle = QAction(QIcon(self.themeSelector.select(':/icons/configure.svg')), self.tr('Point style'), self)
-    actStyle.triggered.connect(self.onPointStyle)
+    self.actMarker.setChecked(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.MarkerRole))
+    self.actCaption.setChecked(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.CaptionRole))
+    self.actSplit.setChecked(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.SplitLineRole))
+    self.actNeglect.setChecked(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.NeglectRole))
+    self.actSkip.setChecked(not TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.IncludeRole))
 
     menu = QMenu(self)
-    menu.addAction(actMarker)
-    menu.addAction(actCaption)
-    menu.addAction(actSplit)
+    menu.addAction(self.actMarker)
+    menu.addAction(self.actCaption)
+    menu.addAction(self.actSplit)
+    menu.addAction(self.actNeglect)
     menu.addSeparator()
-    menu.addAction(actSkip)
+    menu.addAction(self.actSkip)
     menu.addSeparator()
-    menu.addAction(actStyle)
+    if type(self.selectedElement) == CaptionItem:
+      menu.addAction(self.actRename)
+      menu.addAction(self.actResetName)
+      menu.addSeparator()
+    menu.addAction(self.actStyle)
     menu.addSeparator()
 
     actShowGoogleMap = QAction(QIcon(':/icons/googlemaps.png'), self.tr('Google Maps'), self)
@@ -417,25 +520,28 @@ class PlotCanvas(QCustomPlot):
 
   @pyqtSlot()
   def onMarkerPoints(self):
-    checked = self.sender().isChecked()
-    TheDocument.wptmodel.setMarkerStates([self.selectedElement.idx], checked)
-    if checked:
-      index = [m.idx for m in self.infoMarkers].index(self.selectedElement.idx)
-      self.markers += [self.infoMarkers.pop(index)]
-    else:
-      index = [m.idx for m in self.markers].index(self.selectedElement.idx)
-      self.infoMarkers += [self.markers.pop(index)]
-      if not self.showInfo:
-        self.infoMarkers[-1].setVisible(False)
-        self.infoMarkers[-1].setSelectable(False)
+    if self.selectedElement is None:
+      return
 
+    marked = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.MarkerRole)
+    TheDocument.wptmodel.setMarkerStates([self.selectedElement.idx], not marked)
+    index = [m.idx for m in self.markers].index(self.selectedElement.idx)
+    if not marked:
+      self.markers[index].setVisible(True)
+      self.markers[index].setSelectable(True)
+    elif not self.showInfo:
+      self.markers[index].setVisible(False)
+      self.markers[index].setSelectable(False)
     self.replot()
 
   @pyqtSlot()
   def onCaptionPoints(self):
-    checked = self.sender().isChecked()
-    TheDocument.wptmodel.setCaptionStates([self.selectedElement.idx], checked)
-    if checked:
+    if self.selectedElement is None:
+      return
+
+    captioned = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.CaptionRole)
+    TheDocument.wptmodel.setCaptionStates([self.selectedElement.idx], not captioned)
+    if not captioned:
       self.captions += [CaptionItem(self, self.selectedElement.idx, self.selectedElement.posX, self.selectedElement.posY)]
     else:
       for i, c in enumerate(self.captions):
@@ -447,9 +553,12 @@ class PlotCanvas(QCustomPlot):
 
   @pyqtSlot()
   def onSplitLines(self):
-    checked = self.sender().isChecked()
-    TheDocument.wptmodel.setSplitLines([self.selectedElement.idx], checked)
-    if checked:
+    if self.selectedElement is None:
+      return
+
+    splited = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.SplitLineRole)
+    TheDocument.wptmodel.setSplitLines([self.selectedElement.idx], not splited)
+    if not splited:
       self.splitLines += [SplitLine(self, self.selectedElement.idx, self.selectedElement.posX, self.selectedElement.posY)]
     else:
       for i, l in enumerate(self.splitLines):
@@ -460,11 +569,43 @@ class PlotCanvas(QCustomPlot):
     self.replot()
 
   @pyqtSlot()
-  def onSkipPoints(self):
-    checked = self.sender().isChecked()
-    TheDocument.wptmodel.setIncludeStates([self.selectedElement.idx], not checked)
+  def onNeglectDistance(self):
+    if self.selectedElement is None:
+      return
+
+    neglected = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.NeglectRole)
+    TheDocument.wptmodel.setNeglectStates([self.selectedElement.idx], not neglected)
     self.plotProfile(self.column, self.wptRows, self.trkRows, False)
 
+  @pyqtSlot()
+  def onSkipPoints(self):
+    if self.selectedElement is None:
+      return
+
+    included = TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data(gpx.IncludeRole)
+    TheDocument.wptmodel.setIncludeStates([self.selectedElement.idx], not included)
+    self.plotProfile(self.column, self.wptRows, self.trkRows, False)
+
+  @pyqtSlot()
+  def onRenamePoints(self):
+    if type(self.selectedElement) != CaptionItem:
+      return
+
+    oldName = self.selectedElement.text()
+    name, ok = QInputDialog.getText(self, self.tr('Rename waypoint'),
+                                    self.tr('Enter new name for waypoint:'), QLineEdit.Normal, oldName)
+
+    if ok and len(name) > 0:
+      TheDocument.wptmodel.setData(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME), name, Qt.EditRole)
+      self.selectedElement.setText(name)
+      self.replot()
+
+  @pyqtSlot()
+  def onResetPointNames(self):
+    TheDocument.wptmodel.resetNames([self.selectedElement.idx])
+    self.selectedElement.setText(TheDocument.wptmodel.index(self.selectedElement.idx, gpx.NAME).data())
+    self.replot()
+
   def onPointStyle(self):
     dlg = PointConfigDialog(self, self.selectedElement.idx)
     if dlg.exec_() == QDialog.Accepted:
@@ -515,6 +656,9 @@ class PlotCanvas(QCustomPlot):
     else:
       self.selectedElement = None
 
+    if self.showInfo:
+      self.updateLegend()
+
   profileChanged = pyqtSignal()
 
 
@@ -605,6 +749,8 @@ class MarkerItem(QCPGraph):
 
   def updateStyle(self):
     scatterStyle = QCPScatterStyle()
+    selectedScatterStyle = QCPScatterStyle()
+
     if self.idx in TheDocument.wptmodel.getMarkedPoints():
       markerStyle = TheDocument.wptmodel.index(self.idx, gpx.NAME).data(gpx.MarkerStyleRole)
       scatterStyle.setCustomPath(gpx.markerPath(markerStyle[gpx.MARKER_STYLE], 2 * markerStyle[gpx.MARKER_SIZE]))  # for better compatibility with matplotlib
@@ -613,17 +759,14 @@ class MarkerItem(QCPGraph):
         scatterStyle.setPen(QPen(QColor.fromRgba(markerStyle[gpx.MARKER_COLOR]), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
       else:
         scatterStyle.setPen(QPen(QColor.fromRgba(markerStyle[gpx.MARKER_COLOR]), 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+      selectedScatterStyle.setCustomPath(gpx.markerPath(markerStyle[gpx.MARKER_STYLE], 3 * markerStyle[gpx.MARKER_SIZE]))
+      selectedScatterStyle.setPen(QPen(QGuiApplication.palette().highlight().color(), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+
     else:
       scatterStyle.setShape(QCPScatterStyle.ssDisc)
       scatterStyle.setSize(3 * TheConfig.getValue('ProfileStyle', 'ProfileWidth'))
       scatterStyle.setBrush(QColor.fromRgba(TheConfig.getValue('ProfileStyle', 'ProfileColor')))
       scatterStyle.setPen(QPen(QColor.fromRgba(TheConfig.getValue('ProfileStyle', 'ProfileColor')), 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
-
-    selectedScatterStyle = QCPScatterStyle()
-    if self.idx in TheDocument.wptmodel.getMarkedPoints():
-      selectedScatterStyle.setCustomPath(gpx.markerPath(markerStyle[gpx.MARKER_STYLE], 3 * markerStyle[gpx.MARKER_SIZE]))
-      selectedScatterStyle.setPen(QPen(QGuiApplication.palette().highlight().color(), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
-    else:
       selectedScatterStyle.setShape(QCPScatterStyle.ssCircle)
       selectedScatterStyle.setSize(6 * TheConfig.getValue('ProfileStyle', 'ProfileWidth'))
       selectedScatterStyle.setPen(QPen(QGuiApplication.palette().highlight().color(), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
